/*
    Copyright (C) 2007    Dmitry Ivanov, vonami@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARTIMAGENG_IMAGER
#define PARTIMAGENG_IMAGER

#include <sys/types.h>

#include "partimage-ng.h"
#include "partition.h"

namespace PartimageNG {
        class Imager;
}

class PartimageNG::Imager {

private:

        struct Vector {
                u_int64_t nblock;
                u_int16_t count;
        };

        struct VectorCtx {
                u_int16_t count;
                u_int16_t size;
                struct Vector *vectors;
        };

        enum RestorePartitionMode {
                UseExistingPartition,
                CreateNewPartition
        };

public:

        Imager();
        ~Imager();

        void setSaveMbr(bool saveMbr);
        bool saveMbr() const;
        void setGrubHack(bool grubHack);
        bool grubHack() const;
        void setSaveFullSwap(bool saveFullSwap);
        bool saveFullSwap() const;
        void setDesiredBuffering(u_int16_t blocksCount);
        u_int16_t desiredBuffering() const;
        void setDescription(const char *description);
        const char *description() const;
        void setIOErrorsHandling(IOErrorsHandling ioErrorsHandling);
        IOErrorsHandling ioErrorsHandling() const;

        int savePartition(const char *ifile, const char *ofile);
        int restorePartition(const char *ifile, const char *ofile);
        int saveDisk(const char *ifile, const char *ofile);
        int restoreDisk(const char *ifile, const char *ofile);
        int imageinfo(const char *imgfile) const;

private:

        void allocateBuffer(size_t blockSize);
        u_int64_t vectorize(const Bitmap *bitmap, u_int64_t start, u_int16_t count);
        int readVector(int devd, size_t blockSize);
        int writeVector(int outd, size_t blockSize);

        int savePartitionSnapshot(Partition *part, int imaged);
        int restorePartitionSnapshot(int imaged, const char *ofile, RestorePartitionMode mode);

        void dumpVectorCtx();

private:

        size_t m_bufferSize;
        u_int8_t *m_buffer;
        VectorCtx m_vectorCtx;

        u_int16_t m_desiredBuffering;
        bool m_saveMbr;
        bool m_grubHack;
        bool m_saveFullSwap;
        char *m_description;
        size_t m_descriptionSize;
        IOErrorsHandling m_ioErrorsHandling;
};

#endif /* PARTIMAGENG_IMAGER */
